@@ -166,6 +166,7 @@ File.getSaveFileName = function(parentWidget, caption, path, filterStrings) {
     fileDialog.fileMode = QFileDialog.AnyFile;
     fileDialog.acceptMode = QFileDialog.AcceptSave;
 
+    filterStrings = translateFilterStrings(filterStrings);
     fileDialog.setNameFilters(filterStrings);
     
     fileDialog.selectFile(fiDir.completeBaseName());
@@ -234,11 +235,18 @@ File.getSaveFileName = function(parentWidget, caption, path, filterStrings) {
  *
  * \return path and file name chosen by the user
  */
-File.getOpenFileName = function(parentWidget, caption, dir, filterStrings) {
+File.getOpenFileName = function(parentWidget, caption, dir, filterStrings, noAllFiles) {
     if (isNull(filterStrings)) {
         filterStrings = RFileImporterRegistry.getFilterStrings();
+        filterStrings = translateFilterStrings(filterStrings);
+    }
+    if (isNull(noAllFiles)) {
+        noAllFiles = false;
+    }
+
+    if (!noAllFiles) {
+        filterStrings = new Array(qsTr("All Files") + " (*)").concat(filterStrings);
     }
-    filterStrings = new Array(qsTr("All Files") + " (*)").concat(filterStrings);
 
     var fileDialog = new QFileDialog(parentWidget, caption, dir, "");
     var allFilter = filterStrings[0];
