@@ -24,7 +24,8 @@
  * \brief This module contains ECMAScript implementations of various projection (isometric, orthographic) tools.
  */
 
-include("../Modify/Transform.js");
+include("scripts/Modify/Explode/Explode.js");
+include("scripts/Modify/Transform.js");
 include("scripts/ShapeAlgorithms.js");
 
 /**
@@ -44,6 +45,7 @@ function Projection(guiAction) {
     // segmented into line segments:
     this.segmentation = false;
     this.segmentLength = 1.0;
+    this.retainTexts = false;
 
     // force flat projection (for aux constructions):
     this.flat = false;
@@ -228,14 +230,61 @@ Projection.prototype.postTransform = function(entity) {
 /**
  * Callback function for Transform.getOperation. Generic for all projections.
  * \param k Ignored
+ * \param rec Recursive call (don't call preTransform to move / rotate)
  */
-Projection.prototype.transform = function(entity, k, op, preview, flags) {
-    var i;
+Projection.prototype.transform = function(entity, k, op, preview, flags, rec) {
+    if (isNull(rec)) {
+        rec = false;
+    }
 
-    this.preTransform(entity);
+    var i, shapes;
 
-    // shapes that represent this entity:
-    var shapes = entity.getShapes();
+    if (rec!==true) {
+        this.preTransform(entity);
+    }
+
+    if (this.retainTexts && isTextBasedEntity(entity)) {
+        // don't explode text, only project position:
+        var e = entity.clone();
+        e.setSelected(false);
+        var p = e.getAlignmentPoint();
+        var ph = p.operator_add(new RVector(0,e.getTextHeight()));
+        //qDebug("p before:", p);
+        //qDebug("ap:", e.getAlignmentPoint());
+        this.project(p);
+        this.project(ph);
+
+        // project height:
+        //qDebug("p after:", p);
+        //e.setPosition(p);
+
+        e.setAlignmentPoint(p);
+        e.setTextHeight(p.getDistanceTo(ph));
+
+        flags = flags | RAddObjectsOperation.UseAttributes;
+        op.addObject(e, flags);
+        return;
+    }
+
+    // explode block reference to keep attributes (colors, etc.):
+    if (isBlockReferenceEntity(entity)) {
+        var explodedEntities = Explode.explodeEntity(entity);
+        if (!isNull(explodedEntities)) {
+            for (i=0; i<explodedEntities.length; i++) {
+                var explodedEntity = explodedEntities[i];
+
+                if (isEntity(explodedEntity)) {
+                    this.transform(explodedEntity, k, op, preview, flags, true);
+                }
+                else {
+                    qWarning("shape in exploded block");
+                }
+            }
+        }
+        return;
+    }
+
+    shapes = entity.getShapes();
     this.addTransformedShapes(entity, shapes, op, preview, flags);
 
     // text data that is part of this entity (dimension label):
@@ -275,19 +324,35 @@ Projection.prototype.addTransformedShapes = function(entity, shapes, op, preview
                     debugger;
                 }
 
+                if (isPolylineShape(s[n])) {
+                    s[n].simplify(0.001);
+                }
+
                 hd.addBoundary(s[n], true);
             }
         }
 
         var e = entity.clone();
         e.setData(hd);
+        e.setSelected(false);
 
-        flags = flags | RAddObjectsOperation.UseAttributes;
+//        for (i=0; i<hd.getLoopCount(); i++) {
+//            s = hd.getLoopBoundary(i);
+//            qDebug("hatch boundary:", s);
+//        }
+
+        //e.getData().getDocument().getStorage().setObjectId(e, RObject.INVALID_ID);
+        flags = flags | RAddObjectsOperation.UseAttributes | RAddObjectsOperation.ForceNew;
+        //flags = flags | RAddObjectsOperation.UseAttributes;
         op.addObject(e, flags);
         return;
     }
 
     for (i=0; i<shapes.length; i++) {
+        if (isNull(shapes[i])) {
+            continue;
+        }
+
         s = this.projectShape(shapes[i].data(), preview);
         for (n=0; n<s.length; n++) {
             if (isNull(s[n])) {
@@ -328,7 +393,7 @@ Projection.prototype.trimShape = function(shape) {
  * chosen reference point.
  */
 Projection.prototype.project = function(p) {
-    return p;
+    return;
 }
 
 /**
@@ -345,12 +410,15 @@ Projection.prototype.isFlip = function() {
  *
  * \return Array of new projected shapes.
  */
-Projection.prototype.projectShape = function(shape, preview, trim) {
+Projection.prototype.projectShape = function(shape, preview, trim, rec) {
     var p, p2, i, k, s, ret;
 
     if (isNull(trim)) {
         trim = true;
     }
+    if (isNull(rec)) {
+        rec = false;
+    }
 
     // trim shape and project resulting segments:
     if (trim===true && (isPointShape(shape) || isLineShape(shape) || isArcShape(shape))) {
@@ -386,10 +454,13 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
             }
 
             var polyline = ShapeAlgorithms.lineOrArcToPolyline(shape, numSegments);
-            var projectedPl = this.projectShape(polyline, preview, trim)[0];
-            if (!isNull(projectedPl)) {
-                projectedPl.simplify();
-                ret.push(projectedPl);
+            var projectedPls = this.projectShape(polyline, preview, trim, true);
+            if (projectedPls.length>0) {
+                var projectedPl = projectedPls[0];
+                if (!isNull(projectedPl)) {
+                    projectedPl.simplify(0.01);
+                    ret.push(projectedPl);
+                }
             }
             this.segmentation = true;
 
@@ -432,12 +503,20 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
 
     if (isArcShape(shape)) {
         var self=this;
+        // TODO: use RShape::transformArc
+        // make RShapeTransformation scriptable
+        // override RShapeTransformation::transform
         var ellipse = ShapeAlgorithms.transformArc(shape,
             function(p) {
                 self.project(p);
             }
         );
-        return [ ellipse ];
+        if (ellipse.isValid()) {
+            return [ ellipse ];
+        }
+        else {
+            return [];
+        }
     }
 
     if (isCircleShape(shape)) {
@@ -463,6 +542,7 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
             }
         }
 
+        // create polyline from ellipse:
         var ple = ShapeAlgorithms.approximateEllipse(shape, 16);
         return this.projectShape(ple, preview, trim);
     }
@@ -499,6 +579,11 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
     }
 
     if (isPolylineShape(shape)) {
+        if (!rec) {
+            // remove duplicate vertices (zero length segments):
+            shape.normalize(RS.PointTolerance);
+        }
+
         // add polyline, split up at arcs that are projected into ellipses and
         // at gaps caused by trimming:
         ret = [];
@@ -509,7 +594,8 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
         // project each segment individually:
         for (i=0; i<segmentCount; i++) {
             s = shape.getSegmentAt(i).data();
-            s = this.projectShape(s, preview, trim);
+
+            s = this.projectShape(s, preview, trim, true);
 
             // iterate through projected shapes (trimming might lead to gaps):
             for (k=0; k<s.length; k++) {
@@ -531,6 +617,7 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
                 // s is an ellipse:
                 var gotEllipse = isEllipseShape(seg);
 
+                // reached last segment of current polyline:
                 if (i===segmentCount-1 || gotEllipse || gotGap) {
                     if (pl.countVertices()===1) {
                         // don't add polyline for one single shape:
@@ -542,7 +629,10 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
                     }
 
                     if (gotEllipse) {
-                        ret.push(seg);
+                        if (seg.isValid()) {
+                            ret.push(seg);
+                        }
+
                         pl = new RPolyline();
                     }
                     else if (gotGap){
@@ -559,6 +649,35 @@ Projection.prototype.projectShape = function(shape, preview, trim) {
             ret.push(pl);
         }
 
+        // set global width if polyline has uniform global semgent width:
+        var gotGlobalWidth = false;
+        var startWidths = shape.getStartWidths();
+        var endWidths = shape.getEndWidths();
+        if (startWidths.length>0 && endWidths.length>0) {
+            if (startWidths[0]>RS.PointTolerance) {
+                startWidths.sortNumerical();
+                endWidths.sortNumerical();
+                if (RMath.fuzzyCompare(startWidths[0], startWidths[startWidths.length-1]) &&
+                    RMath.fuzzyCompare(endWidths[0], endWidths[endWidths.length-1]) &&
+                    RMath.fuzzyCompare(startWidths[0], endWidths[0])) {
+
+                    // got global polyline width:
+                    gotGlobalWidth = true;
+                    for (i=0; i<ret.length; i++) {
+                        ret[i].setGlobalWidth(startWidths[0]);
+                    }
+                }
+            }
+        }
+
+        // set segment widths if there's no segmentation and the number of segments match:
+        if (!gotGlobalWidth && !this.segmentation && ret.length===1 && shape.countSegments()===ret[0].countSegments()) {
+            for (i=0; i<shape.countVertices(); i++) {
+                ret[0].setStartWidthAt(i, shape.getStartWidthAt(i));
+                ret[0].setEndWidthAt(i, shape.getEndWidthAt(i));
+            }
+        }
+
         return ret;
     }
 
@@ -583,6 +702,12 @@ Projection.prototype.slotSegmentLengthChanged = function(l) {
     this.updatePreview(true);
 };
 
+Projection.prototype.slotRetainTextsChanged = function(v) {
+    this.retainTexts = v;
+    this.clearCache();
+    this.updatePreview(true);
+};
+
 Projection.init = function() {
     Projection.getMenu();
     Projection.getToolBar();
